@@ -308,9 +308,8 @@ afs_AllocCBR(void)
 	    afs_stats_cmperf.CallBackFlushes++;
 	} else {
 	    /* try allocating */
-	    tsp =
-		(struct afs_cbr *)afs_osi_Alloc(AFS_NCBRS *
-						sizeof(struct afs_cbr));
+	    tsp = afs_osi_Alloc(AFS_NCBRS * sizeof(struct afs_cbr));
+	    osi_Assert(tsp != NULL);
 	    for (i = 0; i < AFS_NCBRS - 1; i++) {
 		tsp[i].next = &tsp[i + 1];
 	    }
@@ -350,6 +349,39 @@ afs_FreeCBR(struct afs_cbr *asp)
     return 0;
 }
 
+static void
+FlushAllVCBs(struct rx_connection **rxconns, int nconns, int nservers,
+	     struct afs_conn **conns, struct srvAddr **addrs)
+{
+    afs_int32 *results;
+    afs_int32 i;
+
+    results = afs_osi_Alloc(nservers * sizeof (afs_int32));
+    osi_Assert(results != NULL);
+
+    AFS_GUNLOCK();
+    multi_Rx(rxconns,nconns)
+    {
+        multi_RXAFS_GiveUpAllCallBacks();
+        results[multi_i] = multi_error;
+    } multi_End;
+    AFS_GLOCK();
+
+    /*
+     * Freeing the CBR will unlink it from the server's CBR list
+     * do it here, not in the loop, because a dynamic CBR will call
+     * into the memory management routines.
+     */
+    for ( i = 0 ; i < nconns ; i++ ) {
+	if (results[i] == 0) {
+	    /* Unchain all of them */
+	    while (addrs[i]->server->cbrs)
+		afs_FreeCBR(addrs[i]->server->cbrs);
+	}
+    }
+    afs_osi_Free(results, nservers * sizeof(afs_int32));
+}
+
 /*!
  *   Flush all queued callbacks to all servers.
  *
@@ -374,13 +406,27 @@ afs_FlushVCBs(afs_int32 lockit)
     struct afs_conn *tc;
     int safety1, safety2, safety3;
     XSTATS_DECLS;
+
+    if (AFS_IS_DISCONNECTED)
+	return ENETDOWN;
+
     if ((code = afs_InitReq(&treq, afs_osi_credp)))
 	return code;
     treq.flags |= O_NONBLOCK;
     tfids = afs_osi_Alloc(sizeof(struct AFSFid) * AFS_MAXCBRSCALL);
+    osi_Assert(tfids != NULL);
 
     if (lockit)
 	ObtainWriteLock(&afs_xvcb, 273);
+    /*
+     * Shutting down.
+     * First, attempt a multi across everything, all addresses
+     * for all servers we know of.
+     */
+
+    if (lockit == 2)
+	afs_LoopServers(AFS_LS_ALL, NULL, 0, FlushAllVCBs, NULL);
+
     ObtainReadLock(&afs_xserver);
     for (i = 0; i < NSERVERS; i++) {
 	for (safety1 = 0, tsp = afs_servers[i];
@@ -410,7 +456,7 @@ afs_FlushVCBs(afs_int32 lockit)
 		    for (safety3 = 0; safety3 < AFS_MAXHOSTS * 2; safety3++) {
 			tc = afs_ConnByHost(tsp, tsp->cell->fsport,
 					    tsp->cell->cellNum, &treq, 0,
-					    SHARED_LOCK, 0, &rxconn);
+					    SHARED_LOCK, &rxconn);
 			if (tc) {
 			    XSTATS_START_TIME
 				(AFS_STATS_FS_RPCIDX_GIVEUPCALLBACKS);
@@ -1532,7 +1578,6 @@ afs_RemoteLookup(struct VenusFid *afid, struct vrequest *areq,
 		 struct AFSVolSync *tsyncp)
 {
     afs_int32 code;
-    afs_uint32 start;
     struct afs_conn *tc;
     struct rx_connection *rxconn;
     struct AFSFetchStatus OutDirStatus;
@@ -1543,8 +1588,7 @@ afs_RemoteLookup(struct VenusFid *afid, struct vrequest *areq,
 	tc = afs_Conn(afid, areq, SHARED_LOCK, &rxconn);
 	if (tc) {
 	    if (serverp)
-		*serverp = tc->srvr->server;
-	    start = osi_Time();
+		*serverp = tc->parent->srvr->server;
 	    XSTATS_START_TIME(AFS_STATS_FS_RPCIDX_XLOOKUP);
 	    RX_AFS_GUNLOCK();
 	    code =
@@ -2322,7 +2366,7 @@ afs_FetchStatus(struct vcache * avc, struct VenusFid * afid,
 	tc = afs_Conn(afid, areq, SHARED_LOCK, &rxconn);
 	avc->dchint = NULL;	/* invalidate hints */
 	if (tc) {
-	    avc->callback = tc->srvr->server;
+	    avc->callback = tc->parent->srvr->server;
 	    start = osi_Time();
 	    XSTATS_START_TIME(AFS_STATS_FS_RPCIDX_FETCHSTATUS);
 	    RX_AFS_GUNLOCK();
@@ -2919,7 +2963,8 @@ afs_vcacheInit(int astatSize)
 
 #if !defined(AFS_LINUX22_ENV)
     /* Allocate and thread the struct vcache entries */
-    tvp = (struct vcache *)afs_osi_Alloc(astatSize * sizeof(struct vcache));
+    tvp = afs_osi_Alloc(astatSize * sizeof(struct vcache));
+    osi_Assert(tvp != NULL);
     memset(tvp, 0, sizeof(struct vcache) * astatSize);
 
     Initial_freeVCList = tvp;
@@ -3085,7 +3130,7 @@ afs_DisconGiveUpCallbacks(void)
 
     ReleaseWriteLock(&afs_xvcache);
 
-    afs_FlushVCBs(1);
+    afs_FlushVCBs(2);
 }
 
 /*!
