@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -10,37 +10,22 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
+#include <afs/procmgmt.h>
+#include <roken.h>
 
 #include <stddef.h>
-#include <stdlib.h>
-#include <stdio.h>
-#include <dirent.h>
-#include <errno.h>
-#include <sys/types.h>
-#ifdef AFS_NT40_ENV
-#include <io.h>
-#else
-#include <sys/file.h>
-#include <sys/time.h>
-#endif
-#ifdef BOZO_SAVE_CORES
-#include <time.h>
-#endif
-#include <sys/stat.h>
-#include <string.h>
 
-#include <afs/procmgmt.h>	/* signal(), kill(), wait(), etc. */
 #include <lwp.h>
 #include <rx/rx.h>
 #include <afs/audit.h>
 #include <afs/afsutil.h>
 #include <afs/fileutil.h>
+
 #include "bnode.h"
 #include "bosprototypes.h"
 
-#if defined(AFS_AIX_ENV) || defined(AFS_SUN4_ENV)
-/* All known versions of AIX lack WCOREDUMP but this works */
-#define WCOREDUMP(x) ((x) & 0x80)
+#ifndef WCOREDUMP
+#define WCOREDUMP(x) ((x) & 0200)
 #endif
 
 #define BNODE_LWP_STACKSIZE	(16 * 1024)
@@ -55,6 +40,7 @@ static struct bnode_stats {
     int weirdPids;
 } bnode_stats;
 
+extern const char *DoCore;
 #ifndef AFS_NT40_ENV
 extern char **environ;		/* env structure */
 #endif
@@ -63,9 +49,9 @@ int hdl_notifier(struct bnode_proc *tp);
 
 /* Remember the name of the process, if any, that failed last */
 static void
-RememberProcName(register struct bnode_proc *ap)
+RememberProcName(struct bnode_proc *ap)
 {
-    register struct bnode *tbnodep;
+    struct bnode *tbnodep;
 
     tbnodep = ap->bnode;
     if (tbnodep->lastErrorName) {
@@ -82,9 +68,14 @@ RememberProcName(register struct bnode_proc *ap)
  * be stored.
  */
 int
-bnode_CoreName(register struct bnode *abnode, char *acoreName, char *abuffer)
+bnode_CoreName(struct bnode *abnode, char *acoreName, char *abuffer)
 {
-    strcpy(abuffer, AFSDIR_SERVER_CORELOG_FILEPATH);
+    if (DoCore) {
+	strcpy(abuffer, DoCore);
+	strcat(abuffer, "/");
+	strcat(abuffer, AFSDIR_CORE_FILE);
+    } else
+	strcpy(abuffer, AFSDIR_SERVER_CORELOG_FILEPATH);
     if (acoreName) {
 	strcat(abuffer, acoreName);
 	strcat(abuffer, ".");
@@ -95,12 +86,12 @@ bnode_CoreName(register struct bnode *abnode, char *acoreName, char *abuffer)
 
 /* save core file, if any */
 static void
-SaveCore(register struct bnode *abnode, register struct bnode_proc
+SaveCore(struct bnode *abnode, struct bnode_proc
 	 *aproc)
 {
     char tbuffer[256];
     struct stat tstat;
-    register afs_int32 code;
+    afs_int32 code = 0;
     char *corefile = NULL;
 #ifdef BOZO_SAVE_CORES
     struct timeval Start;
@@ -110,14 +101,23 @@ SaveCore(register struct bnode *abnode, register struct bnode_proc
 
     /* Linux always appends the PID to core dumps from threaded processes, so
      * we have to scan the directory to find core files under another name. */
-    code = stat(AFSDIR_SERVER_CORELOG_FILEPATH, &tstat);
+    if (DoCore) {
+	strcpy(tbuffer, DoCore);
+	strcat(tbuffer, "/");
+	strcat(tbuffer, AFSDIR_CORE_FILE);
+    } else
+	code = stat(AFSDIR_SERVER_CORELOG_FILEPATH, &tstat);
     if (code) {
         DIR *logdir;
         struct dirent *file;
         size_t length;
         unsigned long pid;
+	const char *coredir = AFSDIR_LOGS_DIR;
 
-        logdir = opendir(AFSDIR_LOGS_DIR);
+	if (DoCore)
+	  coredir = DoCore;
+
+	logdir = opendir(coredir);
         if (logdir == NULL)
             return;
         while ((file = readdir(logdir)) != NULL) {
@@ -125,19 +125,20 @@ SaveCore(register struct bnode *abnode, register struct bnode_proc
                 continue;
             pid = atol(file->d_name + 5);
             if (pid == aproc->pid) {
-                length = strlen(AFSDIR_LOGS_DIR) + strlen(file->d_name) + 2;
+                length = strlen(coredir) + strlen(file->d_name) + 2;
                 corefile = malloc(length);
                 if (corefile == NULL) {
                     closedir(logdir);
                     return;
                 }
-                snprintf(corefile, length, "%s/%s", AFSDIR_LOGS_DIR,
-                         file->d_name);
+                snprintf(corefile, length, "%s/%s", coredir, file->d_name);
                 code = 0;
                 break;
             }
         }
         closedir(logdir);
+    } else {
+	corefile = strdup(tbuffer);
     }
     if (code)
 	return;
@@ -151,42 +152,38 @@ SaveCore(register struct bnode *abnode, register struct bnode_proc
 	    TimeFields->tm_hour, TimeFields->tm_min, TimeFields->tm_sec);
     strcpy(tbuffer, FileName);
 #endif
-    if (corefile == NULL)
-        code = renamefile(AFSDIR_SERVER_CORELOG_FILEPATH, tbuffer);
-    else {
-        code = renamefile(corefile, tbuffer);
-        free(corefile);
-    }
+    code = renamefile(corefile, tbuffer);
+    free(corefile);
 }
 
 int
-bnode_GetString(register struct bnode *abnode, register char *abuffer,
-		register afs_int32 alen)
+bnode_GetString(struct bnode *abnode, char *abuffer,
+		afs_int32 alen)
 {
     return BOP_GETSTRING(abnode, abuffer, alen);
 }
 
 int
-bnode_GetParm(register struct bnode *abnode, register afs_int32 aindex,
-	      register char *abuffer, afs_int32 alen)
+bnode_GetParm(struct bnode *abnode, afs_int32 aindex,
+	      char *abuffer, afs_int32 alen)
 {
     return BOP_GETPARM(abnode, aindex, abuffer, alen);
 }
 
 int
-bnode_GetStat(register struct bnode *abnode, register afs_int32 * astatus)
+bnode_GetStat(struct bnode *abnode, afs_int32 * astatus)
 {
     return BOP_GETSTAT(abnode, astatus);
 }
 
 int
-bnode_RestartP(register struct bnode *abnode)
+bnode_RestartP(struct bnode *abnode)
 {
     return BOP_RESTARTP(abnode);
 }
 
 static int
-bnode_Check(register struct bnode *abnode)
+bnode_Check(struct bnode *abnode)
 {
     if (abnode->flags & BNODE_WAIT) {
 	abnode->flags &= ~BNODE_WAIT;
@@ -197,7 +194,7 @@ bnode_Check(register struct bnode *abnode)
 
 /* tell if an instance has a core file */
 int
-bnode_HasCore(register struct bnode *abnode)
+bnode_HasCore(struct bnode *abnode)
 {
     return BOP_HASCORE(abnode);
 }
@@ -206,8 +203,8 @@ bnode_HasCore(register struct bnode *abnode)
 int
 bnode_WaitAll(void)
 {
-    register struct bnode *tb;
-    register afs_int32 code;
+    struct bnode *tb;
+    afs_int32 code;
     afs_int32 stat;
 
   retry:
@@ -231,9 +228,9 @@ bnode_WaitAll(void)
 
 /* wait until bnode status is correct */
 int
-bnode_WaitStatus(register struct bnode *abnode, int astatus)
+bnode_WaitStatus(struct bnode *abnode, int astatus)
 {
-    register afs_int32 code;
+    afs_int32 code;
     afs_int32 stat;
 
     bnode_Hold(abnode);
@@ -259,7 +256,7 @@ bnode_WaitStatus(register struct bnode *abnode, int astatus)
 }
 
 int
-bnode_SetStat(register struct bnode *abnode, register int agoal)
+bnode_SetStat(struct bnode *abnode, int agoal)
 {
     abnode->goal = agoal;
     bnode_Check(abnode);
@@ -269,7 +266,7 @@ bnode_SetStat(register struct bnode *abnode, register int agoal)
 }
 
 int
-bnode_SetGoal(register struct bnode *abnode, register int agoal)
+bnode_SetGoal(struct bnode *abnode, int agoal)
 {
     abnode->goal = agoal;
     bnode_Check(abnode);
@@ -277,7 +274,7 @@ bnode_SetGoal(register struct bnode *abnode, register int agoal)
 }
 
 int
-bnode_SetFileGoal(register struct bnode *abnode, register int agoal)
+bnode_SetFileGoal(struct bnode *abnode, int agoal)
 {
     if (abnode->fileGoal == agoal)
 	return 0;		/* already done */
@@ -290,8 +287,8 @@ bnode_SetFileGoal(register struct bnode *abnode, register int agoal)
 int
 bnode_ApplyInstance(int (*aproc) (struct bnode *tb, void *), void *arock)
 {
-    register struct bnode *tb, *nb;
-    register afs_int32 code;
+    struct bnode *tb, *nb;
+    afs_int32 code;
 
     for (tb = allBnodes; tb; tb = nb) {
 	nb = tb->next;
@@ -303,9 +300,9 @@ bnode_ApplyInstance(int (*aproc) (struct bnode *tb, void *), void *arock)
 }
 
 struct bnode *
-bnode_FindInstance(register char *aname)
+bnode_FindInstance(char *aname)
 {
-    register struct bnode *tb;
+    struct bnode *tb;
 
     for (tb = allBnodes; tb; tb = tb->next) {
 	if (!strcmp(tb->name, aname))
@@ -315,9 +312,9 @@ bnode_FindInstance(register char *aname)
 }
 
 static struct bnode_type *
-FindType(register char *aname)
+FindType(char *aname)
 {
-    register struct bnode_type *tt;
+    struct bnode_type *tt;
 
     for (tt = allTypes; tt; tt = tt->next) {
 	if (!strcmp(tt->name, aname))
@@ -329,7 +326,7 @@ FindType(register char *aname)
 int
 bnode_Register(char *atype, struct bnode_ops *aprocs, int anparms)
 {
-    register struct bnode_type *tt;
+    struct bnode_type *tt;
 
     for (tt = allTypes; tt; tt = tt->next) {
 	if (!strcmp(tt->name, atype))
@@ -386,7 +383,7 @@ bnode_Create(char *atype, char *ainstance, struct bnode ** abp, char *ap1,
     *abp = tb;
     tb->type = type;
 
-    /* The fs_create above calls bnode_InitBnode() which always sets the 
+    /* The fs_create above calls bnode_InitBnode() which always sets the
      ** fileGoal to BSTAT_NORMAL .... overwrite it with whatever is passed into
      ** this function as a parameter... */
     tb->fileGoal = fileGoal;
@@ -402,7 +399,7 @@ bnode_Create(char *atype, char *ainstance, struct bnode ** abp, char *ap1,
 int
 bnode_DeleteName(char *ainstance)
 {
-    register struct bnode *tb;
+    struct bnode *tb;
 
     tb = bnode_FindInstance(ainstance);
     if (!tb)
@@ -412,14 +409,14 @@ bnode_DeleteName(char *ainstance)
 }
 
 int
-bnode_Hold(register struct bnode *abnode)
+bnode_Hold(struct bnode *abnode)
 {
     abnode->refCount++;
     return 0;
 }
 
 int
-bnode_Release(register struct bnode *abnode)
+bnode_Release(struct bnode *abnode)
 {
     abnode->refCount--;
     if (abnode->refCount == 0 && abnode->flags & BNODE_DELETE) {
@@ -430,10 +427,10 @@ bnode_Release(register struct bnode *abnode)
 }
 
 int
-bnode_Delete(register struct bnode *abnode)
+bnode_Delete(struct bnode *abnode)
 {
-    register afs_int32 code;
-    register struct bnode **lb, *ub;
+    afs_int32 code;
+    struct bnode **lb, *ub;
     afs_int32 temp;
 
     if (abnode->refCount != 0) {
@@ -466,14 +463,14 @@ bnode_Delete(register struct bnode *abnode)
 
 /* function to tell if there's a timeout coming up */
 int
-bnode_PendingTimeout(register struct bnode *abnode)
+bnode_PendingTimeout(struct bnode *abnode)
 {
     return (abnode->flags & BNODE_NEEDTIMEOUT);
 }
 
 /* function called to set / clear periodic bnode wakeup times */
 int
-bnode_SetTimeout(register struct bnode *abnode, afs_int32 atimeout)
+bnode_SetTimeout(struct bnode *abnode, afs_int32 atimeout)
 {
     if (atimeout != 0) {
 	abnode->nextTimeout = FT_ApproxTime() + atimeout;
@@ -488,7 +485,7 @@ bnode_SetTimeout(register struct bnode *abnode, afs_int32 atimeout)
 
 /* used by new bnode creation code to format bnode header */
 int
-bnode_InitBnode(register struct bnode *abnode, struct bnode_ops *abnodeops,
+bnode_InitBnode(struct bnode *abnode, struct bnode_ops *abnodeops,
 		char *aname)
 {
     struct bnode **lb, *nb;
@@ -512,9 +509,9 @@ bnode_InitBnode(register struct bnode *abnode, struct bnode_ops *abnodeops,
 }
 
 static int
-DeleteProc(register struct bnode_proc *abproc)
+DeleteProc(struct bnode_proc *abproc)
 {
-    register struct bnode_proc **pb, *tb;
+    struct bnode_proc **pb, *tb;
     struct bnode_proc *nb;
 
     for (pb = &allProcs, tb = *pb; tb; pb = &tb->next, tb = nb) {
@@ -532,10 +529,10 @@ DeleteProc(register struct bnode_proc *abproc)
 static void *
 bproc(void *unused)
 {
-    register afs_int32 code;
-    register struct bnode *tb;
-    register afs_int32 temp;
-    register struct bnode_proc *tp;
+    afs_int32 code;
+    struct bnode *tb;
+    afs_int32 temp;
+    struct bnode_proc *tp;
     struct bnode *nb;
     int options;		/* must not be register */
     struct timeval tv;
@@ -683,11 +680,11 @@ bproc(void *unused)
 }
 
 static afs_int32
-SendNotifierData(register int fd, register struct bnode_proc *tp)
+SendNotifierData(int fd, struct bnode_proc *tp)
 {
-    register struct bnode *tb = tp->bnode;
+    struct bnode *tb = tp->bnode;
     char buffer[1000], *bufp = buffer, *buf1;
-    register int len;
+    int len;
 
     /*
      * First sent out the bnode_proc struct
@@ -760,7 +757,7 @@ int
 hdl_notifier(struct bnode_proc *tp)
 {
 #ifndef AFS_NT40_ENV		/* NT notifier callout not yet implemented */
-    int code, pid;
+    int pid;
     struct stat tstat;
 
     if (stat(tp->bnode->notifier, &tstat)) {
@@ -771,16 +768,15 @@ hdl_notifier(struct bnode_proc *tp)
     if ((pid = fork()) == 0) {
 	FILE *fout;
 	struct bnode *tb = tp->bnode;
-	int ec;
 
 #if defined(AFS_HPUX_ENV) || defined(AFS_SUN5_ENV) || defined(AFS_SGI51_ENV)
-	ec = setsid();
+	setsid();
 #elif defined(AFS_DARWIN90_ENV)
-	ec = setpgid(0, 0);
-#elif defined(AFS_LINUX20_ENV) || defined(AFS_AIX_ENV)  
-	ec = setpgrp();
+	setpgid(0, 0);
+#elif defined(AFS_LINUX20_ENV) || defined(AFS_AIX_ENV)
+	setpgrp();
 #else
-	ec = setpgrp(0, 0);
+	setpgrp(0, 0);
 #endif
 	fout = popen(tb->notifier, "w");
 	if (fout == NULL) {
@@ -789,7 +785,7 @@ hdl_notifier(struct bnode_proc *tp)
 	    perror(tb->notifier);
 	    exit(1);
 	}
-	code = SendNotifierData(fileno(fout), tp);
+	SendNotifierData(fileno(fout), tp);
 	pclose(fout);
 	exit(0);
     } else if (pid < 0) {
@@ -818,10 +814,10 @@ bnode_SoftInt(void *param)
 void
 bnode_Int(int asignal)
 {
-    if (asignal == SIGQUIT) {
-	IOMGR_SoftSig(bozo_ShutdownAndExit, (void *) asignal);
+    if (asignal == SIGQUIT || asignal == SIGTERM) {
+	IOMGR_SoftSig(bozo_ShutdownAndExit, (void *)(intptr_t)asignal);
     } else {
-	IOMGR_SoftSig(bnode_SoftInt, (void *) asignal);
+	IOMGR_SoftSig(bnode_SoftInt, (void *)(intptr_t)asignal);
     }
 }
 
@@ -831,7 +827,7 @@ int
 bnode_Init(void)
 {
     PROCESS junk;
-    register afs_int32 code;
+    afs_int32 code;
     struct sigaction newaction;
     static int initDone = 0;
 
@@ -846,12 +842,15 @@ bnode_Init(void)
 			     "bnode-manager", &bproc_pid);
     if (code)
 	return code;
-    memset((char *)&newaction, 0, sizeof(newaction));
+    memset(&newaction, 0, sizeof(newaction));
     newaction.sa_handler = bnode_Int;
     code = sigaction(SIGCHLD, &newaction, NULL);
     if (code)
 	return errno;
     code = sigaction(SIGQUIT, &newaction, NULL);
+    if (code)
+	return errno;
+    code = sigaction(SIGTERM, &newaction, NULL);
     if (code)
 	return errno;
     return code;
@@ -859,9 +858,9 @@ bnode_Init(void)
 
 /* free token list returned by parseLine */
 int
-bnode_FreeTokens(register struct bnode_token *alist)
+bnode_FreeTokens(struct bnode_token *alist)
 {
-    register struct bnode_token *nlist;
+    struct bnode_token *nlist;
     for (; alist; alist = nlist) {
 	nlist = alist->next;
 	free(alist->key);
@@ -883,11 +882,11 @@ int
 bnode_ParseLine(char *aline, struct bnode_token **alist)
 {
     char tbuffer[256];
-    register char *tptr = NULL;
+    char *tptr = NULL;
     int inToken;
     struct bnode_token *first, *last;
-    register struct bnode_token *ttok;
-    register int tc;
+    struct bnode_token *ttok;
+    int tc;
 
     inToken = 0;		/* not copying token chars at start */
     first = (struct bnode_token *)0;
@@ -929,7 +928,6 @@ bnode_ParseLine(char *aline, struct bnode_token **alist)
 	    return 0;
 	}
     }
-    return 0;
 }
 
 #define	MAXVARGS	    128
@@ -983,9 +981,9 @@ bnode_NewProc(struct bnode *abnode, char *aexecString, char *coreName,
 }
 
 int
-bnode_StopProc(register struct bnode_proc *aproc, int asignal)
+bnode_StopProc(struct bnode_proc *aproc, int asignal)
 {
-    register int code;
+    int code;
     if (!(aproc->flags & BPROC_STARTED) || (aproc->flags & BPROC_EXITED))
 	return BZNOTACTIVE;
 
@@ -998,9 +996,9 @@ bnode_StopProc(register struct bnode_proc *aproc, int asignal)
 }
 
 int
-bnode_Deactivate(register struct bnode *abnode)
+bnode_Deactivate(struct bnode *abnode)
 {
-    register struct bnode **pb, *tb;
+    struct bnode **pb, *tb;
     struct bnode *nb;
     if (!(abnode->flags & BNODE_ACTIVE))
 	return BZNOTACTIVE;
