@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -9,59 +9,53 @@
 
 #include <afsconfig.h>
 #include <afs/param.h>
+#include <afs/stds.h>
+
+#include <afs/procmgmt.h>
+#include <roken.h>
 
 #ifdef IGNORE_SOME_GCC_WARNINGS
 # pragma GCC diagnostic warning "-Wdeprecated-declarations"
 #endif
 
-#include <afs/stds.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <fcntl.h>
-#include <errno.h>
-#include <string.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <stdarg.h>
+#ifdef HAVE_SYS_RESOURCE_H
+#include <sys/resource.h>
+#endif
+
 #ifdef AFS_NT40_ENV
-#include <winsock2.h>
 #include <direct.h>
-#include <io.h>
 #include <WINNT/afsevent.h>
-#else
-#include <unistd.h>
-#include <netinet/in.h>
-#include <syslog.h>
 #endif /* AFS_NT40_ENV */
-#include <afs/cellconfig.h>
+
 #include <rx/rx.h>
 #include <rx/xdr.h>
 #include <rx/rx_globals.h>
-#include "bosint.h"
-#include "bnode.h"
-#include "bosprototypes.h"
 #include <rx/rxkad.h>
 #include <rx/rxstat.h>
 #include <afs/keys.h>
 #include <afs/ktime.h>
 #include <afs/afsutil.h>
 #include <afs/fileutil.h>
-#include <afs/procmgmt.h>	/* signal(), kill(), wait(), etc. */
 #include <afs/audit.h>
 #include <afs/cellconfig.h>
+
 #if defined(AFS_SGI_ENV)
 #include <afs/afs_args.h>
 #endif
 
+#include "bosint.h"
+#include "bnode.h"
+#include "bosprototypes.h"
+
 #define BOZO_LWP_STACKSIZE	16000
 extern struct bnode_ops fsbnode_ops, dafsbnode_ops, ezbnode_ops, cronbnode_ops;
 
 struct afsconf_dir *bozo_confdir = 0;	/* bozo configuration dir */
 static PROCESS bozo_pid;
-struct rx_securityClass *bozo_rxsc[3];
 const char *bozo_fileName;
 FILE *bozo_logFile;
 
+const char *DoCore;
 int DoLogging = 0;
 int DoSyslog = 0;
 #ifndef AFS_NT40_ENV
@@ -78,7 +72,6 @@ int rxkadDisableDotCheck = 0;
 #define ADDRSPERSITE 16         /* Same global is in rx/rx_user.c */
 afs_uint32 SHostAddrs[ADDRSPERSITE];
 
-#ifdef BOS_RESTRICTED_MODE
 int bozo_isrestricted = 0;
 int bozo_restdisable = 0;
 
@@ -89,7 +82,6 @@ bozo_insecureme(int sig)
     bozo_isrestricted = 0;
     bozo_restdisable = 1;
 }
-#endif
 
 struct bztemp {
     FILE *file;
@@ -169,7 +161,7 @@ static int
 MakeDir(const char *adir)
 {
     struct stat tstat;
-    register afs_int32 code;
+    afs_int32 code;
     if (stat(adir, &tstat) < 0 || (tstat.st_mode & S_IFMT) != S_IFDIR) {
 	int reqPerm;
 	unlink(adir);
@@ -189,7 +181,7 @@ MakeDir(const char *adir)
 
 /* create all the bozo dirs */
 static int
-CreateDirs(void)
+CreateDirs(const char *coredir)
 {
     if ((!strncmp
 	 (AFSDIR_USR_DIRPATH, AFSDIR_CLIENT_ETC_DIRPATH,
@@ -222,14 +214,16 @@ CreateDirs(void)
     symlink(AFSDIR_SERVER_CELLSERVDB_FILEPATH,
 	    AFSDIR_CLIENT_CELLSERVDB_FILEPATH);
 #endif /* AFS_NT40_ENV */
+    if (coredir)
+	MakeDir(coredir);
     return 0;
 }
 
 /* strip the \\n from the end of the line, if it is present */
 static int
-StripLine(register char *abuffer)
+StripLine(char *abuffer)
 {
-    register char *tp;
+    char *tp;
 
     tp = abuffer + strlen(abuffer);	/* starts off pointing at the null  */
     if (tp == abuffer)
@@ -242,12 +236,12 @@ StripLine(register char *abuffer)
 
 /* write one bnode's worth of entry into the file */
 static int
-bzwrite(register struct bnode *abnode, void *arock)
+bzwrite(struct bnode *abnode, void *arock)
 {
-    register struct bztemp *at = (struct bztemp *)arock;
-    register int i;
+    struct bztemp *at = (struct bztemp *)arock;
+    int i;
     char tbuffer[BOZO_BSSIZE];
-    register afs_int32 code;
+    afs_int32 code;
 
     if (abnode->notifier)
 	fprintf(at->file, "bnode %s %s %d %s\n", abnode->type->name,
@@ -272,18 +266,17 @@ bzwrite(register struct bnode *abnode, void *arock)
 int
 ReadBozoFile(char *aname)
 {
-    register FILE *tfile;
+    FILE *tfile;
     char tbuffer[BOZO_BSSIZE];
-    register char *tp;
+    char *tp;
     char *instp, *typep, *notifier, *notp;
-    register afs_int32 code;
+    afs_int32 code;
     afs_int32 ktmask, ktday, kthour, ktmin, ktsec;
     afs_int32 i, goal;
     struct bnode *tb;
     char *parms[MAXPARMS];
-#ifdef BOS_RESTRICTED_MODE
+    char *thisparms[MAXPARMS];
     int rmode;
-#endif
 
     /* rename BozoInit to BosServer for the user */
     if (!aname) {
@@ -305,27 +298,28 @@ ReadBozoFile(char *aname)
 	}
     }
 
-    /* setup default times we want to do restarts */
-    bozo_nextRestartKT.mask = KTIME_HOUR | KTIME_MIN | KTIME_DAY;
-    bozo_nextRestartKT.hour = 4;	/* 4 am */
+    /* don't do server restarts by default */
+    bozo_nextRestartKT.mask = KTIME_NEVER;
+    bozo_nextRestartKT.hour = 0;
     bozo_nextRestartKT.min = 0;
-    bozo_nextRestartKT.day = 0;	/* Sunday */
+    bozo_nextRestartKT.day = 0;
+
+    /* restart processes at 5am if their binaries have changed */
     bozo_nextDayKT.mask = KTIME_HOUR | KTIME_MIN;
     bozo_nextDayKT.hour = 5;
     bozo_nextDayKT.min = 0;
 
     for (code = 0; code < MAXPARMS; code++)
 	parms[code] = NULL;
-    instp = typep = notifier = NULL;
     tfile = (FILE *) 0;
     if (!aname)
 	aname = (char *)bozo_fileName;
     tfile = fopen(aname, "r");
     if (!tfile)
 	return 0;		/* -1 */
-    instp = (char *)malloc(BOZO_BSSIZE);
-    typep = (char *)malloc(BOZO_BSSIZE);
-    notifier = notp = (char *)malloc(BOZO_BSSIZE);
+    instp = malloc(BOZO_BSSIZE);
+    typep = malloc(BOZO_BSSIZE);
+    notifier = notp = malloc(BOZO_BSSIZE);
     while (1) {
 	/* ok, read lines giving parms and such from the file */
 	tp = fgets(tbuffer, sizeof(tbuffer), tfile);
@@ -367,7 +361,7 @@ ReadBozoFile(char *aname)
 	    bozo_nextDayKT.sec = ktsec;
 	    continue;
 	}
-#ifdef BOS_RESTRICTED_MODE
+
 	if (strncmp(tbuffer, "restrictmode", 12) == 0) {
 	    code = sscanf(tbuffer, "restrictmode %d", &rmode);
 	    if (code != 1) {
@@ -381,7 +375,6 @@ ReadBozoFile(char *aname)
 	    bozo_isrestricted = rmode;
 	    continue;
 	}
-#endif
 
 	if (strncmp("bnode", tbuffer, 5) != 0) {
 	    code = -1;
@@ -397,6 +390,8 @@ ReadBozoFile(char *aname)
 	} else if (code == 3)
 	    notifier = NULL;
 
+	memset(thisparms, 0, sizeof(thisparms));
+
 	for (i = 0; i < MAXPARMS; i++) {
 	    /* now read the parms, until we see an "end" line */
 	    tp = fgets(tbuffer, sizeof(tbuffer), tfile);
@@ -414,12 +409,13 @@ ReadBozoFile(char *aname)
 	    if (!parms[i])	/* make sure there's space */
 		parms[i] = (char *)malloc(BOZO_BSSIZE);
 	    strcpy(parms[i], tbuffer + 5);	/* remember the parameter for later */
+	    thisparms[i] = parms[i];
 	}
 
 	/* ok, we have the type and parms, now create the object */
 	code =
-	    bnode_Create(typep, instp, &tb, parms[0], parms[1], parms[2],
-			 parms[3], parms[4], notifier,
+	    bnode_Create(typep, instp, &tb, thisparms[0], thisparms[1],
+			 thisparms[2], thisparms[3], thisparms[4], notifier,
 			 goal ? BSTAT_NORMAL : BSTAT_SHUTDOWN, 0);
 	if (code)
 	    goto fail;
@@ -453,9 +449,9 @@ ReadBozoFile(char *aname)
 int
 WriteBozoFile(char *aname)
 {
-    register FILE *tfile;
+    FILE *tfile;
     char tbuffer[AFSDIR_PATH_MAX];
-    register afs_int32 code;
+    afs_int32 code;
     struct bztemp btemp;
 
     if (!aname)
@@ -466,9 +462,8 @@ WriteBozoFile(char *aname)
     if (!tfile)
 	return -1;
     btemp.file = tfile;
-#ifdef BOS_RESTRICTED_MODE
+
     fprintf(tfile, "restrictmode %d\n", bozo_isrestricted);
-#endif
     fprintf(tfile, "restarttime %d %d %d %d %d\n", bozo_nextRestartKT.mask,
 	    bozo_nextRestartKT.day, bozo_nextRestartKT.hour,
 	    bozo_nextRestartKT.min, bozo_nextRestartKT.sec);
@@ -495,9 +490,9 @@ WriteBozoFile(char *aname)
 }
 
 static int
-bdrestart(register struct bnode *abnode, void *arock)
+bdrestart(struct bnode *abnode, void *arock)
 {
-    register afs_int32 code;
+    afs_int32 code;
 
     if (abnode->fileGoal != BSTAT_NORMAL || abnode->goal != BSTAT_NORMAL)
 	return 0;		/* don't restart stopped bnodes */
@@ -518,7 +513,7 @@ bdrestart(register struct bnode *abnode, void *arock)
 static void *
 BozoDaemon(void *unused)
 {
-    register afs_int32 now;
+    afs_int32 now;
 
     /* now initialize the values */
     bozo_newKTs = 1;
@@ -526,12 +521,11 @@ BozoDaemon(void *unused)
 	IOMGR_Sleep(60);
 	now = FT_ApproxTime();
 
-#ifdef BOS_RESTRICTED_MODE
 	if (bozo_restdisable) {
 	    bozo_Log("Restricted mode disabled by signal\n");
 	    bozo_restdisable = 0;
 	}
-#endif
+
 	if (bozo_newKTs) {	/* need to recompute restart times */
 	    bozo_newKTs = 0;	/* done for a while */
 	    nextRestart = ktime_next(&bozo_nextRestartKT, BOZO_MINSKIP);
@@ -598,7 +592,7 @@ tweak_config(void)
  * fundamental errors occur.
  *
  * This routine requires
- * 
+ *
  * #include <sys/types.h>
  * #include <sys/stat.h>
  * #include <fcntl.h>
@@ -620,7 +614,7 @@ tweak_config(void)
 static void
 background(void)
 {
-    /* 
+    /*
      * A process is a process group leader if its process ID
      * (getpid()) and its process group ID (getpgrp()) are the same.
      */
@@ -719,7 +713,7 @@ int
 main(int argc, char **argv, char **envp)
 {
     struct rx_service *tservice;
-    register afs_int32 code;
+    afs_int32 code;
     struct afsconf_dir *tdir;
     int noAuth = 0;
     int i;
@@ -727,6 +721,8 @@ main(int argc, char **argv, char **envp)
     int rxMaxMTU = -1;
     afs_uint32 host = htonl(INADDR_ANY);
     char *auditFileName = NULL;
+    struct rx_securityClass **securityClasses;
+    afs_int32 numClasses;
 #ifndef AFS_NT40_ENV
     int nofork = 0;
     struct stat sb;
@@ -735,14 +731,14 @@ main(int argc, char **argv, char **envp)
     struct sigaction nsa;
 
     /* for some reason, this permits user-mode RX to run a lot faster.
-     * we do it here in the bosserver, so we don't have to do it 
+     * we do it here in the bosserver, so we don't have to do it
      * individually in each server.
      */
     tweak_config();
 
     /*
-     * The following signal action for AIX is necessary so that in case of a 
-     * crash (i.e. core is generated) we can include the user's data section 
+     * The following signal action for AIX is necessary so that in case of a
+     * crash (i.e. core is generated) we can include the user's data section
      * in the core dump. Unfortunately, by default, only a partial core is
      * generated which, in many cases, isn't too useful.
      */
@@ -753,9 +749,7 @@ main(int argc, char **argv, char **envp)
     sigaction(SIGABRT, &nsa, NULL);
 #endif
     osi_audit_init();
-#ifdef BOS_RESTRICTED_MODE
     signal(SIGFPE, bozo_insecureme);
-#endif
 
 #ifdef AFS_NT40_ENV
     /* Initialize winsock */
@@ -778,6 +772,7 @@ main(int argc, char **argv, char **envp)
 
     /* some path inits */
     bozo_fileName = AFSDIR_SERVER_BOZCONF_FILEPATH;
+    DoCore = AFSDIR_SERVER_LOGS_DIRPATH;
 
     /* initialize the list of dirpaths that the bosserver has
      * an interest in monitoring */
@@ -807,6 +802,11 @@ main(int argc, char **argv, char **envp)
 	} else if (strncmp(argv[code], "-syslog=", 8) == 0) {
 	    DoSyslog = 1;
 	    DoSyslogFacility = atoi(argv[code] + 8);
+	} else if (strncmp(argv[code], "-cores=", 7) == 0) {
+	    if (strcmp((argv[code]+7), "none") == 0)
+		DoCore = 0;
+	    else
+		DoCore = (argv[code]+7);
 	} else if (strcmp(argv[code], "-nofork") == 0) {
 	    nofork = 1;
 	}
@@ -816,11 +816,9 @@ main(int argc, char **argv, char **envp)
 	} else if (strcmp(argv[code], "-enable_process_stats") == 0) {
 	    rx_enableProcessRPCStats();
 	}
-#ifdef BOS_RESTRICTED_MODE
 	else if (strcmp(argv[code], "-restricted") == 0) {
 	    bozo_isrestricted = 1;
 	}
-#endif
 	else if (strcmp(argv[code], "-rxbind") == 0) {
 	    rxBind = 1;
 	}
@@ -829,14 +827,14 @@ main(int argc, char **argv, char **envp)
 	}
 	else if (!strcmp(argv[code], "-rxmaxmtu")) {
 	    if ((code + 1) >= argc) {
-		fprintf(stderr, "missing argument for -rxmaxmtu\n"); 
-		exit(1); 
+		fprintf(stderr, "missing argument for -rxmaxmtu\n");
+		exit(1);
 	    }
 	    rxMaxMTU = atoi(argv[++code]);
-	    if ((rxMaxMTU < RX_MIN_PACKET_SIZE) || 
+	    if ((rxMaxMTU < RX_MIN_PACKET_SIZE) ||
 		(rxMaxMTU > RX_MAX_PACKET_DATA_SIZE)) {
-		printf("rxMaxMTU %d invalid; must be between %d-%lu\n",
-			rxMaxMTU, RX_MIN_PACKET_SIZE, 
+		printf("rxMaxMTU %d invalid; must be between %d-%" AFS_SIZET_FMT "\n",
+			rxMaxMTU, RX_MIN_PACKET_SIZE,
 			RX_MAX_PACKET_DATA_SIZE);
 		exit(1);
 	    }
@@ -863,6 +861,7 @@ main(int argc, char **argv, char **envp)
 		   "[-rxmaxmtu <bytes>] [-rxbind] [-allow-dotted-principals]"
 		   "[-syslog[=FACILITY]] "
 		   "[-enable_peer_stats] [-enable_process_stats] "
+		   "[-cores=<none|path>] \n"
 		   "[-nofork] " "[-help]\n");
 #else
 	    printf("Usage: bosserver [-noauth] [-log] "
@@ -870,6 +869,7 @@ main(int argc, char **argv, char **envp)
 		   "[-audit-interafce <file|sysvmq> (default is file)] "
 		   "[-rxmaxmtu <bytes>] [-rxbind] [-allow-dotted-principals]"
 		   "[-enable_peer_stats] [-enable_process_stats] "
+		   "[-cores=<none|path>] \n"
 		   "[-help]\n");
 #endif
 	    fflush(stdout);
@@ -900,17 +900,20 @@ main(int argc, char **argv, char **envp)
     bnode_Register("cron", &cronbnode_ops, 2);
 
     /* create useful dirs */
-    CreateDirs();
+    CreateDirs(DoCore);
 
     /* chdir to AFS log directory */
-    chdir(AFSDIR_SERVER_LOGS_DIRPATH);
+    if (DoCore)
+	chdir(DoCore);
+    else
+	chdir(AFSDIR_SERVER_LOGS_DIRPATH);
 
 #if 0
     fputs(AFS_GOVERNMENT_MESSAGE, stdout);
     fflush(stdout);
 #endif
 
-    /* go into the background and remove our controlling tty, close open 
+    /* go into the background and remove our controlling tty, close open
        file desriptors
      */
 
@@ -921,7 +924,7 @@ main(int argc, char **argv, char **envp)
 
     if ((!DoSyslog)
 #ifndef AFS_NT40_ENV
-	&& ((lstat(AFSDIR_BOZLOG_FILE, &sb) == 0) && 
+	&& ((lstat(AFSDIR_BOZLOG_FILE, &sb) == 0) &&
 	!(S_ISFIFO(sb.st_mode)))
 #endif
 	) {
@@ -942,11 +945,45 @@ main(int argc, char **argv, char **envp)
 #endif
     }
 
+#if defined(RLIMIT_CORE) && defined(HAVE_GETRLIMIT)
+    {
+      struct rlimit rlp;
+      getrlimit(RLIMIT_CORE, &rlp);
+      if (!DoCore)
+	  rlp.rlim_cur = 0;
+      else
+	  rlp.rlim_max = rlp.rlim_cur = RLIM_INFINITY;
+      setrlimit(RLIMIT_CORE, &rlp);
+      getrlimit(RLIMIT_CORE, &rlp);
+      bozo_Log("Core limits now %d %d\n",(int)rlp.rlim_cur,(int)rlp.rlim_max);
+    }
+#endif
+
     /* Write current state of directory permissions to log file */
     DirAccessOK();
 
+    if (rxBind) {
+	afs_int32 ccode;
+	if (AFSDIR_SERVER_NETRESTRICT_FILEPATH ||
+	    AFSDIR_SERVER_NETINFO_FILEPATH) {
+	    char reason[1024];
+	    ccode = parseNetFiles(SHostAddrs, NULL, NULL,
+	                          ADDRSPERSITE, reason,
+	                          AFSDIR_SERVER_NETINFO_FILEPATH,
+	                          AFSDIR_SERVER_NETRESTRICT_FILEPATH);
+        } else {
+            ccode = rx_getAllAddr(SHostAddrs, ADDRSPERSITE);
+        }
+        if (ccode == 1)
+            host = SHostAddrs[0];
+    }
+
     for (i = 0; i < 10; i++) {
-	code = rx_Init(htons(AFSCONF_NANNYPORT));
+	if (rxBind) {
+	    code = rx_InitHost(host, htons(AFSCONF_NANNYPORT));
+	} else {
+	    code = rx_Init(htons(AFSCONF_NANNYPORT));
+	}
 	if (code) {
 	    bozo_Log("can't initialize rx: code=%d\n", code);
 	    sleep(3);
@@ -1011,14 +1048,8 @@ main(int argc, char **argv, char **envp)
     /* allow super users to manage RX statistics */
     rx_SetRxStatUserOk(bozo_rxstat_userok);
 
-    /* have bcrypt key now */
-
     afsconf_SetNoAuthFlag(tdir, noAuth);
-
-    bozo_rxsc[0] = rxnull_NewServerSecurityObject();
-    bozo_rxsc[1] = (struct rx_securityClass *)0;
-    bozo_rxsc[2] =
-	rxkad_NewServerSecurityObject(0, tdir, afsconf_GetKey, NULL);
+    afsconf_BuildServerSecurityObjects(tdir, &securityClasses, &numClasses);
 
     /* Disable jumbograms */
     rx_SetNoJumbo();
@@ -1027,28 +1058,9 @@ main(int argc, char **argv, char **envp)
 	rx_SetMaxMTU(rxMaxMTU);
     }
 
-    if (rxBind) {
-	afs_int32 ccode;
-        if (AFSDIR_SERVER_NETRESTRICT_FILEPATH || 
-            AFSDIR_SERVER_NETINFO_FILEPATH) {
-            char reason[1024];
-            ccode = parseNetFiles(SHostAddrs, NULL, NULL,
-                                           ADDRSPERSITE, reason,
-                                           AFSDIR_SERVER_NETINFO_FILEPATH,
-                                           AFSDIR_SERVER_NETRESTRICT_FILEPATH);
-        } else 
-	{
-            ccode = rx_getAllAddr(SHostAddrs, ADDRSPERSITE);
-        }
-        if (ccode == 1) 
-            host = SHostAddrs[0];
-    }
-
-    tservice = rx_NewServiceHost(host,  /* port */ 0, /* service id */ 1,
-			     /*service name */ "bozo",
-			     /* security classes */
-			     bozo_rxsc,
-			     /* numb sec classes */ 3, BOZO_ExecuteRequest);
+    tservice = rx_NewServiceHost(host, 0, /* service id */ 1,
+			         "bozo", securityClasses, numClasses,
+				 BOZO_ExecuteRequest);
     rx_SetMinProcs(tservice, 2);
     rx_SetMaxProcs(tservice, 4);
     rx_SetStackSize(tservice, BOZO_LWP_STACKSIZE);	/* so gethostbyname works (in cell stuff) */
@@ -1058,8 +1070,8 @@ main(int argc, char **argv, char **envp)
     }
 
     tservice =
-	rx_NewServiceHost(host, 0, RX_STATS_SERVICE_ID, "rpcstats", bozo_rxsc,
-			  3, RXSTATS_ExecuteRequest);
+	rx_NewServiceHost(host, 0, RX_STATS_SERVICE_ID, "rpcstats",
+			  securityClasses, numClasses, RXSTATS_ExecuteRequest);
     rx_SetMinProcs(tservice, 2);
     rx_SetMaxProcs(tservice, 4);
     rx_StartServer(1);		/* donate this process */
