@@ -1,7 +1,7 @@
 /*
  * Copyright 2006, Sine Nomine Associates and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -16,19 +16,14 @@
 
 #include <afsconfig.h>
 #include <afs/param.h>
-
-
-#include <stdio.h>
-#include <errno.h>
-#include <sys/file.h>
-#include <netdb.h>
-#include <netinet/in.h>
-#include <time.h>
-#include <string.h>
-
 #include <afs/stds.h>
+
+#include <roken.h>
+
+#include <sys/mman.h>
+
 #include <rx/xdr.h>
-#include <afs/assert.h>
+#include <afs/afs_assert.h>
 #include <lwp.h>
 #include <lock.h>
 #include <afs/afsint.h>
@@ -47,45 +42,11 @@
 #include <afs/afsutil.h>
 #include <rx/rx.h>
 #include <afs/cellconfig.h>
-#include <stdlib.h>
 #include "../util/afsutil_prototypes.h"
 #include "../viced/viced.h"
 #include "../viced/host.h"
 #include "../viced/callback.h"
 #include "serialize_state.h"
-#include <fcntl.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <sys/mman.h>
-#include <unistd.h>
-
-/*@+fcnmacros +macrofcndecl@*/
-#ifdef O_LARGEFILE
-#ifdef S_SPLINT_S
-extern off64_t afs_lseek(int FD, off64_t O, int F);
-#endif /*S_SPLINT_S */
-#define afs_lseek(FD, O, F)	lseek64(FD, (off64_t)(O), F)
-#define afs_stat		stat64
-#define afs_fstat		fstat64
-#define afs_open		open64
-#define afs_fopen		fopen64
-#define afs_mmap                mmap64
-#ifdef AFS_AIX_ENV
-extern void * mmap64();  /* ugly hack since aix build env appears to be somewhat broken */
-#endif
-#else /* !O_LARGEFILE */
-#ifdef S_SPLINT_S
-extern off_t afs_lseek(int FD, off_t O, int F);
-#endif /*S_SPLINT_S */
-#define afs_lseek(FD, O, F)	lseek(FD, (off_t)(O), F)
-#define afs_stat		stat
-#define afs_fstat		fstat
-#define afs_open		open
-#define afs_fopen		fopen
-#define afs_mmap                mmap
-#endif /* !O_LARGEFILE */
-/*@=fcnmacros =macrofcndecl@*/
-
 
 #ifndef AFS_DEMAND_ATTACH_FS
 int
@@ -237,10 +198,6 @@ static struct {
     byte cb_valid;
 } cb_cursor;
 
-static struct {
-    void ** cursor;
-} cb_cache;
-
 static void
 usage(char * prog)
 {
@@ -251,7 +208,6 @@ int
 main(int argc, char ** argv)
 {
     banner();
-
     if (argc > 2 || (argc == 2 && !strcmp(argv[1], "-h"))) {
 	usage(argv[0]);
 	return 1;
@@ -263,7 +219,7 @@ main(int argc, char ** argv)
 	if (openFile(argv[1]))
 	    return 1;
     } else {
-	if (openFile(AFSDIR_SERVER_FSSTATE_FILEPATH))
+	if (openFile((char *)AFSDIR_SERVER_FSSTATE_FILEPATH))
 	    return 1;
     }
 
@@ -277,7 +233,7 @@ openFile(char * path)
 {
     int ret = 0;
     struct afs_stat status;
-    
+
     dump_fd = afs_open(path, O_RDWR);
     if (dump_fd == -1) {
 	fprintf(stderr, "dump file '%s' failed to open\n", path);
@@ -302,7 +258,7 @@ openFile(char * path)
 	goto done;
     }
 
-    printf("mapped %d bytes at 0x%x\n", map_len, map);
+    printf("mapped %lu bytes at %"AFS_PTR_FMT"\n", (unsigned long)map_len, map);
 
  done:
     if (ret) {
@@ -373,7 +329,7 @@ prompt(void)
 		fprintf(stderr, "prompt state broken; aborting\n");
 		return;
 	    }
-	    gets(input);
+	    fgets(input, 256, stdin);
 
 	    if (!strcmp(input, "")) {
 		/* repeat last command */
@@ -386,10 +342,10 @@ prompt(void)
 		strlcpy(prev_input, input, sizeof(prev_input));
 	    }
 
-	    tok = strtok(input, " \t");
+	    tok = strtok(input, " \t\n");
 	}
 	while (tok && !strcmp(tok, ";")) {
-	    tok = strtok(NULL, "; \t");
+	    tok = strtok(NULL, "; \t\n");
 	}
 
 	if (!tok) {
@@ -576,7 +532,7 @@ prompt(void)
 	    tok = strtok(NULL, " \t");
 	    if (!tok || strcasecmp(tok, "by")) {
 		tok = NULL;
-		fprintf(stderr, "find syntax error 1 (%s)\n", 
+		fprintf(stderr, "find syntax error 1 (%s)\n",
 			(tok) ? tok : "nil");
 		continue;
 	    }
@@ -742,8 +698,8 @@ print_cb_help(void)
     do { \
         char * _p = (char *)addr; \
         char * _m = (char *)map; \
-        printf("loading structure from address 0x%x (offset %u)\n", \
-               addr, _p-_m); \
+        printf("loading structure from address %"AFS_PTR_FMT" (offset %ld)\n", \
+               addr, (long)(_p-_m)); \
     } while (0)
 
 /* structs */
@@ -1100,7 +1056,7 @@ dump_next_he(void)
 	fprintf(stderr, "no more HEs\n");
 	return;
     }
-    
+
     dump_he(he_cursor.idx+1);
 }
 
@@ -1116,7 +1072,7 @@ dump_next_fe(void)
 	fprintf(stderr, "no more FEs\n");
 	return;
     }
-    
+
     dump_fe(fe_cursor.idx+1);
 }
 
@@ -1132,7 +1088,7 @@ dump_next_cb(void)
 	fprintf(stderr, "no more CBs\n");
 	return;
     }
-    
+
     dump_cb(cb_cursor.idx+1);
 }
 
@@ -1143,7 +1099,7 @@ dump_prev_he(void)
 	fprintf(stderr, "no more HEs\n");
 	return;
     }
-    
+
     dump_he(he_cursor.idx-1);
 }
 
@@ -1154,7 +1110,7 @@ dump_prev_fe(void)
 	fprintf(stderr, "no more FEs\n");
 	return;
     }
-    
+
     dump_fe(fe_cursor.idx-1);
 }
 
@@ -1165,7 +1121,7 @@ dump_prev_cb(void)
 	fprintf(stderr, "no more CBs\n");
 	return;
     }
-    
+
     dump_cb(cb_cursor.idx-1);
 }
 
@@ -1181,7 +1137,7 @@ dump_first_fe(void)
 	fprintf(stderr, "no FEs present\n");
 	return;
     }
-    
+
     dump_fe(0);
 }
 
@@ -1197,7 +1153,7 @@ dump_first_he(void)
 	fprintf(stderr, "no HEs present\n");
 	return;
     }
-    
+
     dump_he(0);
 }
 
@@ -1213,7 +1169,7 @@ dump_first_cb(void)
 	fprintf(stderr, "no CBs present\n");
 	return;
     }
-    
+
     dump_cb(0);
 }
 
@@ -1229,7 +1185,7 @@ dump_last_he(void)
 	fprintf(stderr, "no HEs present\n");
 	return;
     }
-    
+
     dump_he(hdrs.h_hdr.records-1);
 }
 
@@ -1245,7 +1201,7 @@ dump_last_fe(void)
 	fprintf(stderr, "no FEs present\n");
 	return;
     }
-    
+
     dump_fe(hdrs.cb_hdr.nFEs-1);
 }
 
@@ -1283,8 +1239,9 @@ dump_he_hdr(void)
 static void
 dump_he_entry(void)
 {
+    char hoststr[16];
     DPFSO0("hostDiskEntry");
-    DPFS1("host", afs_inet_ntoa(he_cursor.he.host));
+    DPFS1("host", afs_inet_ntoa_r(he_cursor.he.host, hoststr));
     DPFV1("port", "u", he_cursor.he.port);
     DPFX1("hostFlags", he_cursor.he.hostFlags);
     DPFV1("Console", "u", he_cursor.he.Console);
@@ -1312,6 +1269,7 @@ dump_he_interfaces(void)
     char temp_str[40];
     struct Interface * ifp;
     int len, i;
+    char hoststr[16];
 
     if (!he_cursor.hdr.interfaces)
 	return;
@@ -1330,7 +1288,7 @@ dump_he_interfaces(void)
     for (i = 0; i < he_cursor.hdr.interfaces; i++) {
 	snprintf(temp_str, sizeof(temp_str), "interface[%d]", i);
 	DPFSO1(temp_str);
-	DPFS2("addr", afs_inet_ntoa(ifp->interface[i].addr));
+	DPFS2("addr", afs_inet_ntoa_r(ifp->interface[i].addr, hoststr));
 	DPFV2("port", "u", ifp->interface[i].port);
 	DPFSC1;
     }
@@ -1346,7 +1304,6 @@ dump_he_interfaces(void)
 static void
 dump_he_hcps(void)
 {
-    char temp_str[40];
     afs_int32 * hcps;
     int len, i;
 
@@ -1434,14 +1391,13 @@ hexdump_map(afs_uint32 offset, afs_uint32 len)
 {
     int i;
     unsigned char * p = (unsigned char *)map;
-    afs_uint32 c32;
 
     if (!len)
 	return;
 
     if ((offset + len) > map_len) {
-	fprintf(stderr, "offset + length exceeds memory map size (%u > %u)\n",
-		offset+len, map_len);
+	fprintf(stderr, "offset + length exceeds memory map size (%u > %lu)\n",
+		offset+len, (unsigned long)map_len);
 	return;
     }
 
@@ -1497,7 +1453,7 @@ get_h_hdr(void)
 	fprintf(stderr, "hi offset bits set in h_offset; can't get host_state_header\n");
 	return 1;
     }
-    if ((lo >= map_len) || 
+    if ((lo >= map_len) ||
 	((lo + sizeof(struct host_state_header)) > map_len) ||
 	(lo + sizeof(struct host_state_header) < lo)) {
 	fprintf(stderr, "h_offset puts host_state_header beyond end of memory map\n");
@@ -1531,7 +1487,7 @@ get_cb_hdr(void)
 	fprintf(stderr, "hi offset bits set in cb_offset; can't get callback_state_header\n");
 	return 1;
     }
-    if ((lo >= map_len) || 
+    if ((lo >= map_len) ||
 	((lo + sizeof(struct callback_state_header)) > map_len) ||
 	(lo + sizeof(struct callback_state_header) < lo)) {
 	fprintf(stderr, "cb_offset puts callback_state_header beyond end of memory map\n");
@@ -1584,7 +1540,7 @@ get_cb_timeout_hdr(void)
 	fprintf(stderr, "hi offset bits set in timeout_offset; can't get callback_state_timeout_header\n");
 	return 1;
     }
-    if ((lo >= map_len) || 
+    if ((lo >= map_len) ||
 	((lo + sizeof(struct callback_state_timeout_header)) > map_len) ||
 	(lo + sizeof(struct callback_state_timeout_header) < lo)) {
 	fprintf(stderr, "timeout_offset puts callback_state_timeout_header beyond end of memory map\n");
@@ -1605,8 +1561,6 @@ get_cb_timeout_hdr(void)
 static int
 get_cb_timeout(void)
 {
-    char * buf;
-
     if (hdrs.timeout)
 	return 0;
 
@@ -1637,7 +1591,7 @@ get_cb_fehash_hdr(void)
 	fprintf(stderr, "hi offset bits set in fehash_offset; can't get callback_state_fehash_header\n");
 	return 1;
     }
-    if ((lo >= map_len) || 
+    if ((lo >= map_len) ||
 	((lo + sizeof(struct callback_state_fehash_header)) > map_len) ||
 	(lo + sizeof(struct callback_state_fehash_header) < lo)) {
 	fprintf(stderr, "timeout_offset puts callback_state_fehash_header beyond end of memory map\n");
@@ -1658,8 +1612,6 @@ get_cb_fehash_hdr(void)
 static int
 get_cb_fehash(void)
 {
-    char * buf;
-
     if (hdrs.fehash)
 	return 0;
 
@@ -1830,7 +1782,7 @@ get_fe_hdr(void)
 	    return 1;
         }
     }
-    
+
     memcpy(&fe_cursor.hdr, fe_cursor.cursor, sizeof(struct callback_state_entry_header));
     fe_cursor.hdr_valid = 1;
     return 0;
@@ -1861,7 +1813,6 @@ get_fe_entry(void)
 static int
 get_cb(afs_uint32 idx)
 {
-    int i;
     char * p;
 
     if (get_fe(fe_cursor.idx))
@@ -1895,6 +1846,7 @@ get_cb_entry(void)
     return 0;
 }
 
+#if 0
 static int
 find_he_by_index(afs_uint32 idx)
 {
@@ -1919,6 +1871,7 @@ find_he_by_index(afs_uint32 idx)
     }
     return 1;
 }
+#endif
 
 static int
 find_fe_by_index(afs_uint32 idx)
